@@ -3,19 +3,20 @@ DIAMOND protein aligner
 Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
+#include <memory>
 #include "../basic/value.h"
 #include "align.h"
 #include "../data/reference.h"
@@ -25,7 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "query_mapper.h"
 #include "../util/merge_sort.h"
 
-using std::map;
+using namespace std;
 
 DpStat dp_stat;
 
@@ -35,9 +36,9 @@ struct Align_fetcher
 	{
 		it_ = begin;
 		end_ = end;
-		queue_ = auto_ptr<Queue>(new Queue(qbegin, qend));
+		queue_ = unique_ptr<Queue>(new Queue(qbegin, qend));
 	}
-	void operator()(size_t query)
+	bool operator()(size_t query)
 	{
 		const unsigned q = (unsigned)query,
 			c = align_mode.query_contexts;
@@ -46,19 +47,26 @@ struct Align_fetcher
 			++it_;
 		end = it_;
 		this->query = query;
+		target_parallel = (end - begin > config.query_parallel_limit) && config.frame_shift != 0 && align_mode.mode == Align_mode::blastx && config.toppercent < 100 && config.query_range_culling;
+		return target_parallel;
 	}
 	bool get()
 	{
 		return queue_->get(*this) != Queue::end;
 	}
+	void release() {
+		if (target_parallel)
+			queue_->release();
+	}
 	size_t query;
 	vector<hit>::iterator begin, end;
+	bool target_parallel;
 private:	
 	static vector<hit>::iterator it_, end_;
-	static auto_ptr<Queue> queue_;
+	static unique_ptr<Queue> queue_;
 };
 
-auto_ptr<Queue> Align_fetcher::queue_;
+unique_ptr<Queue> Align_fetcher::queue_;
 vector<hit>::iterator Align_fetcher::it_;
 vector<hit>::iterator Align_fetcher::end_;
 
@@ -84,27 +92,31 @@ void align_worker(size_t thread_id, const Parameters *params, const Metadata *me
 		if (config.ext == Config::swipe)
 			mapper = new ExtensionPipeline::Swipe::Pipeline(*params, hits.query, hits.begin, hits.end);
 		else if (config.frame_shift != 0)
-			mapper = new ExtensionPipeline::BandedSwipe::Pipeline(*params, hits.query, hits.begin, hits.end, dp_stat);
+			mapper = new ExtensionPipeline::BandedSwipe::Pipeline(*params, hits.query, hits.begin, hits.end, dp_stat, hits.target_parallel);
 		else
 			mapper = new ExtensionPipeline::Greedy::Pipeline(*params, hits.query, hits.begin, hits.end);
+		task_timer timer("Initializing mapper", hits.target_parallel ? 3 : UINT_MAX);
 		mapper->init();
+		timer.finish();
 		mapper->run(stat);
 
+		timer.go("Generating output");
 		TextBuffer *buf = 0;
 		if (*output_format != Output_format::null) {
 			buf = new TextBuffer;
 			const bool aligned = mapper->generate_output(*buf, stat, *metadata);
-			if (aligned && !config.unaligned.empty())
+			if (aligned && (!config.unaligned.empty() || !config.aligned_file.empty()))
 				query_aligned[hits.query] = true;
 		}
 		delete mapper;
 		OutputSink::get().push(hits.query, buf);
+		hits.release();
 	}
 	statistics += stat;
 	::dp_stat += dp_stat;
 }
 
-void align_queries(Trace_pt_buffer &trace_pts, OutputFile* output_file, const Parameters &params, const Metadata &metadata)
+void align_queries(Trace_pt_buffer &trace_pts, Consumer* output_file, const Parameters &params, const Metadata &metadata)
 {
 	const size_t max_size = (size_t)std::min(config.chunk_size*1e9 * 9 * 2 / config.lowmem, 2e9);
 	pair<size_t, size_t> query_range;
@@ -121,11 +133,11 @@ void align_queries(Trace_pt_buffer &trace_pts, OutputFile* output_file, const Pa
 		v->init();
 		timer.go("Computing alignments");
 		Align_fetcher::init(query_range.first, query_range.second, v->begin(), v->end());
-		OutputSink::instance = auto_ptr<OutputSink>(new OutputSink(query_range.first, output_file));
+		OutputSink::instance = unique_ptr<OutputSink>(new OutputSink(query_range.first, output_file));
 		Thread_pool threads;
-		if (config.verbosity >= 3)
+		if (config.verbosity >= 3 && config.load_balancing == Config::query_parallel)
 			threads.push_back(launch_thread(heartbeat_worker, query_range.second));
-		size_t n_threads = (config.threads_align == 0 ? config.threads_ : config.threads_align);
+		size_t n_threads = config.load_balancing == Config::query_parallel ? (config.threads_align == 0 ? config.threads_ : config.threads_align) : 1;
 		for (size_t i = 0; i < n_threads; ++i)
 			//threads.push_back(launch_thread(static_cast<void(*)(size_t)>(&align_worker), i));
 			threads.push_back(launch_thread(align_worker, i, &params, &metadata));
