@@ -3,19 +3,20 @@ DIAMOND protein aligner
 Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
+#include <memory>
 #include "../util/command_line_parser.h"
 #include "config.h"
 #include "../util/util.h"
@@ -32,6 +33,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../basic/translate.h"
 #include "../dp/dp.h"
 #include "masking.h"
+#include "../util/system/system.h"
+
+using namespace std;
 
 Config config;
 
@@ -63,7 +67,10 @@ Config::Config(int argc, const char **argv)
 		.add_command("db-annot-stats", "")
 		.add_command("read-sim", "")
 		.add_command("info", "")
-		.add_command("seed-stat", "");
+		.add_command("seed-stat", "")
+		.add_command("smith-waterman", "")
+		.add_command("protein-snps", "")
+		.add_command("cluster", "");
 
 	Options_group general("General options");
 	general.add()
@@ -106,8 +113,9 @@ Config::Config(int argc, const char **argv)
 \tstitle means Subject Title\n\
 \tsalltitles means All Subject Title(s), separated by a '<>'\n\
 \tqcovhsp means Query Coverage Per HSP\n\
-\tqtitle means Query title\n\n\
-\tDefault: qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore", output_format)
+\tqtitle means Query title\n\
+\tqqual means Query quality values\n\
+\n\tDefault: qseqid sseqid pident length mismatch gapopen qstart qend sstart send evalue bitscore", output_format)
 ("verbose", 'v', "verbose console output", verbose)
 ("log", 0, "enable debug log", debug_log)
 ("quiet", 0, "disable console output", quiet);
@@ -121,6 +129,7 @@ Config::Config(int argc, const char **argv)
 		("query", 'q', "input query file", query_file)
 		("strand", 0, "query strands to search (both/minus/plus)", query_strands, string("both"))
 		("un", 0, "file for unaligned queries", unaligned)
+		("al", 0, "file or aligned queries", aligned_file)
 		("unal", 0, "report unaligned queries (0=no, 1=yes)", report_unaligned, -1)
 		("max-target-seqs", 'k', "maximum number of target sequences to report alignments for", max_alignments, uint64_t(25))
 		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0)
@@ -139,6 +148,7 @@ Config::Config(int argc, const char **argv)
 		("gapopen", 0, "gap open penalty", gap_open, -1)
 		("gapextend", 0, "gap extension penalty", gap_extend, -1)
 		("frameshift", 'F', "frame shift penalty (default=disabled)", frame_shift)
+		("long-reads", 0, "short for --range-culling --top 10 -F 15", long_reads)
 		("matrix", 0, "score matrix for protein alignment (default=BLOSUM62)", matrix, string("blosum62"))
 		("custom-matrix", 0, "file containing custom scoring matrix", matrix_file)
 		("lambda", 0, "lambda parameter for custom matrix", lambda)
@@ -223,7 +233,7 @@ Config::Config(int argc, const char **argv)
 		("sw", 0, "", use_smith_waterman)
 		("superblock", 0, "", superblock, 128)
 		("max-cells", 0, "", max_cells, 10000000u)
-		("lb", 0, "", load_balancing, (unsigned)Config::query_parallel)
+		("load-balancing", 0, "", load_balancing, (unsigned)Config::query_parallel)
 		("ext", 0, "", ext, (int)Config::greedy)
 		("br", 0, "", benchmark_ranking)
 		("log-query", 0, "", log_query)
@@ -246,11 +256,24 @@ Config::Config(int argc, const char **argv)
 		("hash-join", 0, "", hash_join)
 		("sort-join", 0, "", sort_join)
 		("simple-freq", 0, "", simple_freq)
-		("freq-treshold", 0, "", freq_treshold);
+		("freq-treshold", 0, "", freq_treshold)
+		("filter-locus", 0, "", filter_locus)
+		("use-dataset-field", 0, "", use_dataset_field)
+		("store-query-quality", 0, "", store_query_quality)
+		("swipe-chunk-size", 0, "", swipe_chunk_size, 256u)
+		("query-parallel-limit", 0, "", query_parallel_limit, 1000000u);
 		
 	parser.add(general).add(makedb).add(aligner).add(advanced).add(view_options).add(getseq_options).add(hidden_options);
 	parser.store(argc, argv, command);
 
+	if (long_reads) {
+		query_range_culling = true;
+		if (toppercent == 100.0)
+			toppercent = 10.0;
+		if (frame_shift == 0)
+			frame_shift = 15;
+	}
+
 	switch (command) {
 	case Config::makedb:
 		if (database == "")
@@ -319,12 +342,14 @@ Config::Config(int argc, const char **argv)
 		;
 	}
 
-	for (int i = 0; i < argc; ++i)
-		log_stream << argv[i] << ' ';
-	log_stream << endl;
+	invocation = join(' ', vector<string>(&argv[0], &argv[argc]));
+	log_stream << invocation << endl;
 
 	if (!no_auto_append) {
-		auto_append_extension(database, ".dmnd");
+		if (command == Config::makedb)
+			auto_append_extension(database, ".dmnd");
+		else
+			auto_append_extension_if_exists(database, ".dmnd");
 		if (command == Config::view)
 			auto_append_extension(daa_file, ".daa");
 		if (compression == 1)
@@ -332,7 +357,7 @@ Config::Config(int argc, const char **argv)
 	}
 
 	message_stream << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " | by Benjamin Buchfink <buchfink@gmail.com>" << endl;
-	message_stream << "Licensed under the GNU AGPL <https://www.gnu.org/licenses/agpl.txt>" << endl;
+	message_stream << "Licensed under the GNU GPL <https://www.gnu.org/licenses/gpl.txt>" << endl;
 	message_stream << "Check http://github.com/bbuchfink/diamond for updates." << endl << endl;
 #ifndef NDEBUG
 	verbose_stream << "Assertions enabled." << endl;
@@ -344,6 +369,7 @@ Config::Config(int argc, const char **argv)
 	case Config::blastp:
 	case Config::blastx:
 	case Config::view:
+	case Config::cluster:
 		message_stream << "#CPU threads: " << threads_ << endl;
 	default:
 		;
@@ -357,6 +383,7 @@ Config::Config(int argc, const char **argv)
 	case Config::opt:
 	case Config::mask:
 	case Config::makedb:
+	case Config::cluster:
 		if (frame_shift != 0 && command == Config::blastp)
 			throw std::runtime_error("Frameshift alignments are only supported for translated searches.");
 		if (query_range_culling && frame_shift == 0)
@@ -372,11 +399,11 @@ Config::Config(int argc, const char **argv)
 		}
 		message_stream << "Scoring parameters: " << score_matrix << endl;
 		if (masking == 1)
-			Masking::instance = auto_ptr<Masking>(new Masking(score_matrix));
+			Masking::instance = unique_ptr<Masking>(new Masking(score_matrix));
 	}
 
 	if (command == Config::blastp || command == Config::blastx || command == Config::benchmark || command == Config::model_sim || command == Config::opt
-		|| command == Config::mask) {
+		|| command == Config::mask || command == Config::cluster) {
 		if (tmpdir == "")
 			tmpdir = extract_dir(output_file);
 		
@@ -398,4 +425,6 @@ Config::Config(int argc, const char **argv)
 
 	/*log_stream << "sizeof(hit)=" << sizeof(hit) << " sizeof(packed_uint40_t)=" << sizeof(packed_uint40_t)
 		<< " sizeof(sorted_list::entry)=" << sizeof(sorted_list::entry) << endl;*/
+
+	use_lazy_dict = false;
 }
