@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -26,6 +26,8 @@ Partitioned_histogram query_hst;
 vector<bool> query_aligned;
 Seed_set *query_seeds = 0;
 Hashed_seed_set *query_seeds_hashed = 0;
+String_set<0> *query_qual = nullptr;
+vector<unsigned> query_block_to_database_id;
 
 void write_unaligned(OutputFile *file)
 {
@@ -40,4 +42,19 @@ void write_unaligned(OutputFile *file)
 			buf.clear();
 		}
 	}
+}
+
+void write_aligned(OutputFile *file)
+{
+	const size_t n = query_ids::get().get_length();
+	TextBuffer buf;
+	for (size_t i = 0; i < n; ++i) {
+		if (query_aligned[i]) {
+			buf << '>' << query_ids::get()[i].c_str() << '\n';
+			(align_mode.query_translated ? query_source_seqs::get()[i] : query_seqs::get()[i]).print(buf, input_value_traits);
+			buf << '\n';
+			file->write(buf.get_begin(), buf.size());
+			buf.clear();
+		}
+	}
 }
\ No newline at end of file
