@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <iostream>
 #include <set>
 #include <map>
+#include <memory>
 #include "../basic/config.h"
 #include "reference.h"
 #include "load_seqs.h"
@@ -39,8 +40,7 @@ unsigned current_ref_block;
 Sequence_set* ref_seqs::data_ = 0;
 bool blocked_processing;
 
-using std::cout;
-using std::endl;
+using namespace std;
 
 Serializer& operator<<(Serializer &s, const ReferenceHeader2 &h)
 {
@@ -74,8 +74,7 @@ struct Pos_record
 	uint32_t seq_len;
 };
 
-DatabaseFile::DatabaseFile():
-	InputFile(config.database, InputFile::BUFFERED)
+void DatabaseFile::init()
 {
 	read_header(*this, ref_header);
 	*this >> header2;
@@ -86,6 +85,27 @@ DatabaseFile::DatabaseFile():
 	pos_array_offset = ref_header.pos_array_offset;
 }
 
+DatabaseFile::DatabaseFile(const string &input_file):
+	InputFile(input_file, InputFile::BUFFERED),
+	temporary(false)
+{
+	init();
+}
+
+DatabaseFile::DatabaseFile(TempFile &tmp_file):
+	InputFile(tmp_file, 0),
+	temporary(true)
+{
+	init();
+}
+
+void DatabaseFile::close() {
+	if (temporary)
+		InputFile::close_and_delete();
+	else
+		InputFile::close();
+}
+
 void DatabaseFile::read_header(InputFile &stream, ReferenceHeader &header)
 {
 	if (stream.read(&header, 1) != 1)
@@ -112,7 +132,7 @@ void DatabaseFile::rewind()
 
 void push_seq(const sequence &seq, const sequence &id, uint64_t &offset, vector<Pos_record> &pos_array, OutputFile &out, size_t &letters, size_t &n_seqs)
 {	
-	pos_array.push_back(Pos_record(offset, seq.length()));
+	pos_array.emplace_back(offset, seq.length());
 	out.write("\xff", 1);
 	out.write(seq.data(), seq.length());
 	out.write("\xff", 1);
@@ -122,7 +142,7 @@ void push_seq(const sequence &seq, const sequence &id, uint64_t &offset, vector<
 	offset += seq.length() + id.length() + 3;
 }
 
-void make_db()
+void make_db(TempFile **tmp_out)
 {
 	message_stream << "Database file: " << config.input_ref_file << endl;
 	
@@ -131,17 +151,17 @@ void make_db()
 	if (config.input_ref_file == "")
 		std::cerr << "Input file parameter (--in) is missing. Input will be read from stdin." << endl;
 	task_timer timer("Opening the database file", true);
-	auto_ptr<TextInputFile> db_file (new TextInputFile(config.input_ref_file));
+	unique_ptr<TextInputFile> db_file (new TextInputFile(config.input_ref_file));
 	
-	OutputFile out(config.database);
+	OutputFile *out = tmp_out ? new TempFile() : new OutputFile(config.database);
 	ReferenceHeader header;
 	ReferenceHeader2 header2;
 
-	out.write(&header, 1);
-	out << header2;
+	out->write(&header, 1);
+	*out << header2;
 
 	size_t letters = 0, n = 0, n_seqs = 0;
-	uint64_t offset = out.tell();
+	uint64_t offset = out->tell();
 
 	Sequence_set *seqs;
 	String_set<0> *ids;
@@ -150,7 +170,7 @@ void make_db()
 	FileBackedBuffer accessions;
 
 	try {
-		while ((timer.go("Loading sequences"), n = load_seqs(*db_file, format, &seqs, ids, 0, (size_t)(1e9), string())) > 0) {
+		while ((timer.go("Loading sequences"), n = load_seqs(*db_file, format, &seqs, ids, 0, nullptr, (size_t)(1e9), string())) > 0) {
 			if (config.masking == 1) {
 				timer.go("Masking sequences");
 				mask_seqs(*seqs, Masking::get(), false);
@@ -160,7 +180,7 @@ void make_db()
 				sequence seq = (*seqs)[i];
 				if (seq.length() == 0)
 					throw std::runtime_error("File format error: sequence of length 0 at line " + to_string(db_file->line_count));
-				push_seq(seq, (*ids)[i], offset, pos_array, out, letters, n_seqs);
+				push_seq(seq, (*ids)[i], offset, pos_array, *out, letters, n_seqs);
 			}
 			if (!config.prot_accession2taxid.empty()) {
 				timer.go("Writing accessions");
@@ -178,8 +198,8 @@ void make_db()
 		}
 	}
 	catch (std::exception&) {
-		out.close();
-		out.remove();
+		out->close();
+		out->remove();
 		throw;
 	}
 
@@ -187,19 +207,19 @@ void make_db()
 	
 	timer.go("Writing trailer");
 	header.pos_array_offset = offset;
-	pos_array.push_back(Pos_record(offset, 0));
-	out.write_raw(pos_array);
+	pos_array.emplace_back(offset, 0);
+	out->write_raw(pos_array);
 	timer.finish();
 
 	taxonomy.init();
 	if (!config.prot_accession2taxid.empty()) {
-		header2.taxon_array_offset = out.tell();
-		TaxonList::build(out, accessions.rewind(), n_seqs);
-		header2.taxon_array_size = out.tell() - header2.taxon_array_offset;
+		header2.taxon_array_offset = out->tell();
+		TaxonList::build(*out, accessions.rewind(), n_seqs);
+		header2.taxon_array_size = out->tell() - header2.taxon_array_offset;
 	}
 	if (!config.nodesdmp.empty()) {
-		header2.taxon_nodes_offset = out.tell();
-		TaxonomyNodes::build(out);
+		header2.taxon_nodes_offset = out->tell();
+		TaxonomyNodes::build(*out);
 	}
 
 	timer.go("Closing the input file");
@@ -208,10 +228,15 @@ void make_db()
 	timer.go("Closing the database file");
 	header.letters = letters;
 	header.sequences = n_seqs;
-	out.seek(0);
-	out.write(&header, 1);
-	out << header2;
-	out.close();
+	out->seek(0);
+	out->write(&header, 1);
+	*out << header2;
+	if (tmp_out) {
+		*tmp_out = static_cast<TempFile*>(out);
+	} else {
+		out->close();
+		delete out;
+	}
 
 	timer.finish();
 	message_stream << "Database hash = " << hex_print(header2.hash, 16) << endl;
@@ -219,17 +244,29 @@ void make_db()
 	message_stream << "Total time = " << total.getElapsedTimeInSec() << "s" << endl;
 }
 
-bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, size_t max_letters, bool masked, bool load_ids, const vector<bool> *filter)
+void DatabaseFile::seek_seq(size_t i) {
+	pos_array_offset = ref_header.pos_array_offset + sizeof(Pos_record)*i;
+}
+
+size_t DatabaseFile::tell_seq() const {
+	return (pos_array_offset - ref_header.pos_array_offset) / sizeof(Pos_record);
+}
+
+void DatabaseFile::seek_direct() {
+	seek(sizeof(ReferenceHeader) + sizeof(ReferenceHeader2) + 8);
+}
+
+bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, size_t max_letters, bool masked, Sequence_set **dst_seq, String_set<0> **dst_id, bool load_ids, const vector<bool> *filter)
 {
 	task_timer timer("Loading reference sequences");
 	seek(pos_array_offset);
-	size_t database_id = (pos_array_offset - ref_header.pos_array_offset) / sizeof(Pos_record);
+	size_t database_id = tell_seq();
 	size_t letters = 0, seqs = 0, id_letters = 0, seqs_processed = 0;
 	vector<uint64_t> filtered_pos;
 	block_to_database_id.clear();
 
-	ref_seqs::data_ = new Sequence_set;
-	if(load_ids) ref_ids::data_ = new String_set<0>;
+	*dst_seq = new Sequence_set;
+	if(load_ids) *dst_id = new String_set<0>;
 
 	Pos_record r;
 	read(&r, 1);
@@ -241,10 +278,10 @@ bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, size_t max_
 		read(&r_next, 1);
 		if (!filter || (*filter)[database_id]) {
 			letters += r.seq_len;
-			ref_seqs::data_->reserve(r.seq_len);
+			(*dst_seq)->reserve(r.seq_len);
 			const size_t id_len = r_next.pos - r.pos - r.seq_len - 3;
 			id_letters += id_len;
-			if (load_ids) ref_ids::data_->reserve(id_len);
+			if (load_ids) (*dst_id)->reserve(id_len);
 			++seqs;
 			block_to_database_id.push_back((unsigned)database_id);
 			if (filter) filtered_pos.push_back(last ? 0 : r.pos);
@@ -259,36 +296,36 @@ bool DatabaseFile::load_seqs(vector<unsigned> &block_to_database_id, size_t max_
 	}
 
 	if (seqs == 0) {
-		delete ref_seqs::data_;
-		ref_seqs::data_ = NULL;
-		if (load_ids) delete ref_ids::data_;
-		ref_ids::data_ = NULL;
+		delete (*dst_seq);
+		(*dst_seq) = NULL;
+		if (load_ids) delete (*dst_id);
+		(*dst_id) = NULL;
 		return false;
 	}
 
-	ref_seqs::data_->finish_reserve();
-	if(load_ids) ref_ids::data_->finish_reserve();
+	(*dst_seq)->finish_reserve();
+	if(load_ids) (*dst_id)->finish_reserve();
 	seek(start_offset);
 	size_t masked_letters = 0;
 
 	for (size_t n = 0; n < seqs; ++n) {
 		if (filter && filtered_pos[n]) seek(filtered_pos[n]);
-		read(ref_seqs::data_->ptr(n) - 1, ref_seqs::data_->length(n) + 2);
-		*(ref_seqs::data_->ptr(n) - 1) = sequence::DELIMITER;
-		*(ref_seqs::data_->ptr(n) + ref_seqs::data_->length(n)) = sequence::DELIMITER;
+		read((*dst_seq)->ptr(n) - 1, (*dst_seq)->length(n) + 2);
+		*((*dst_seq)->ptr(n) - 1) = sequence::DELIMITER;
+		*((*dst_seq)->ptr(n) + (*dst_seq)->length(n)) = sequence::DELIMITER;
 		if (load_ids)
-			read(ref_ids::data_->ptr(n), ref_ids::data_->length(n) + 1);
+			read((*dst_id)->ptr(n), (*dst_id)->length(n) + 1);
 		else
 			if (!seek_forward('\0')) throw std::runtime_error("Unexpected end of file.");
 		if (config.masking == 1 && masked)
-			Masking::get().bit_to_hard_mask(ref_seqs::data_->ptr(n), ref_seqs::data_->length(n), masked_letters);
+			Masking::get().bit_to_hard_mask((*dst_seq)->ptr(n), (*dst_seq)->length(n), masked_letters);
 		else
-			Masking::get().remove_bit_mask(ref_seqs::data_->ptr(n), ref_seqs::data_->length(n));
-		if (!config.sfilt.empty() && strstr(ref_ids::get()[n].c_str(), config.sfilt.c_str()) == 0)
-			memset(ref_seqs::data_->ptr(n), value_traits.mask_char, ref_seqs::data_->length(n));
+			Masking::get().remove_bit_mask((*dst_seq)->ptr(n), (*dst_seq)->length(n));
+		if (!config.sfilt.empty() && strstr((**dst_id)[n].c_str(), config.sfilt.c_str()) == 0)
+			memset((*dst_seq)->ptr(n), value_traits.mask_char, (*dst_seq)->length(n));
 	}
 	timer.finish();
-	ref_seqs::get().print_stats();
+	(*dst_seq)->print_stats();
 	log_stream << "Masked letters = " << masked_letters << endl;
 
 	blocked_processing = seqs_processed < ref_header.sequences;
@@ -362,4 +399,23 @@ void db_info()
 	cout << "Sequences = " << header.sequences << endl;
 	cout << "Letters = " << header.letters << endl;
 	db_file.close();
+}
+
+DatabaseFile* DatabaseFile::auto_create_from_fasta() {
+	InputFile db_file(config.database);
+	uint64_t magic_number;
+	if (db_file.read(&magic_number, 1) != 1 || magic_number != ReferenceHeader::MAGIC_NUMBER) {
+		message_stream << "Database file is not a DIAMOND database, treating as FASTA." << endl;
+		db_file.close();
+		config.input_ref_file = config.database;
+		TempFile *db;
+		make_db(&db);
+		DatabaseFile *r(new DatabaseFile(*db));
+		delete db;
+		return r;
+	}
+	else {
+		db_file.close();
+		return new DatabaseFile(config.database);
+	}
 }
\ No newline at end of file
