@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -34,7 +34,7 @@ using std::string;
 struct ReferenceHeader
 {
 	ReferenceHeader() :
-		magic_number(0x24af8a415ee186dllu),
+		magic_number(MAGIC_NUMBER),
 		build(Const::build_version),
 		db_version(current_db_version),
 		sequences(0),
@@ -43,6 +43,7 @@ struct ReferenceHeader
 	uint64_t magic_number;
 	uint32_t build, db_version;
 	uint64_t sequences, letters, pos_array_offset;
+	static const uint64_t MAGIC_NUMBER = 0x24af8a415ee186dllu;
 	enum { current_db_version = 2 };
 };
 
@@ -70,23 +71,35 @@ struct Database_format_exception : public std::exception
 
 struct DatabaseFile : public InputFile
 {
-	DatabaseFile();
+
+	DatabaseFile(const string &file_name);
+	DatabaseFile(TempFile &tmp_file);
 	static void read_header(InputFile &stream, ReferenceHeader &header);
+	static DatabaseFile* auto_create_from_fasta();
 	void rewind();
-	bool load_seqs(vector<unsigned> &block_to_database_id, size_t max_letters, bool masked, bool load_ids = true, const vector<bool> *filter = NULL);
+	bool load_seqs(vector<unsigned> &block_to_database_id, size_t max_letters, bool masked, Sequence_set **dst_seq, String_set<0> **dst_id, bool load_ids = true, const vector<bool> *filter = NULL);
 	void get_seq();
 	void read_seq(string &id, vector<char> &seq);
 	bool has_taxon_id_lists();
 	bool has_taxon_nodes();
+	void close();
+	void seek_seq(size_t i);
+	size_t tell_seq() const;
+	void seek_direct();
 
 	enum { min_build_required = 74 };
 
+	bool temporary;
 	size_t pos_array_offset;
 	ReferenceHeader ref_header;
 	ReferenceHeader2 header2;
+
+private:
+	void init();
+
 };
 
-void make_db();
+void make_db(TempFile **tmp_out = nullptr);
 
 struct ref_seqs
 {
