@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -295,12 +295,7 @@ struct score_vector<int16_t>
 		data_ = _mm_subs_epi16(data_, rhs.data_);
 		return *this;
 	}
-
-	int16_t operator [](unsigned i) const
-	{
-		return *(((int16_t*)&data_) + i); // ^ 0x8000;
-	}
-
+	
 	score_vector& max(const score_vector &rhs)
 	{
 		data_ = _mm_max_epi16(data_, rhs.data_);
@@ -322,11 +317,6 @@ struct score_vector<int16_t>
 		return _mm_cmpgt_epi16(data_, rhs.data_);
 	}
 
-	operator int() const
-	{
-		return this->operator[](0);
-	}
-
 	__m128i data_;
 
 };
