@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../basic/match.h"
 #include "output_format.h"
 #include "../data/taxonomy.h"
+#include "../data/queries.h"
 
 const char* Blast_tab_format::field_str[] = {
 	"qseqid",		// 0 means Query Seq - id
@@ -70,7 +71,11 @@ const char* Blast_tab_format::field_str[] = {
 	"qtitle",		// 45 means Query title
 	"swdiff",		// 46
 	"time", 		// 47
-	"full_sseq"		// 48
+	"full_sseq",	// 48
+	"qqual",		// 49
+	"qnum",			// 50
+	"snum",			// 51
+	"scovhsp"		// 52
 };
 
 Blast_tab_format::Blast_tab_format() :
@@ -93,6 +98,8 @@ Blast_tab_format::Blast_tab_format() :
 			config.salltitles = true;
 		if (j == 48)
 			config.use_lazy_dict = true;
+		if (j == 49)
+			config.store_query_quality = true;
 	}
 }
 
@@ -242,8 +249,20 @@ void Blast_tab_format::print_match(const Hsp_context& r, const Metadata &metadat
 		case 48:
 			out << r.subject_seq;
 			break;
+		case 49:
+			out << (*query_qual)[r.query_id].c_str();
+			break;
+		case 50:
+			out << query_block_to_database_id[r.query_id];
+			break;
+		case 51:
+			out << r.orig_subject_id;
+			break;
+		case 52:
+			out << (double)r.subject_range().length() * 100.0 / r.subject_len;
+			break;
 		default:
-			throw std::runtime_error("Invalid output field");
+			throw std::runtime_error(string("Invalid output field: ") + field_str[*i]);
 		}
 		if (i < fields.end() - 1)
 			out << '\t';
@@ -298,8 +317,11 @@ void Blast_tab_format::print_query_intro(size_t query_num, const char *query_nam
 			case 45:
 				out << query_name;
 				break;
+			case 49:
+				out << (*query_qual)[query_num].c_str();
+				break;
 			default:
-				throw std::runtime_error("Invalid output field");
+				throw std::runtime_error(string("Invalid output field: ") + field_str[*i]);
 			}
 			if (i < fields.end() - 1)
 				out << '\t';			
