@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define OUTPUT_FORMAT_H_
 
 #include <exception>
+#include <memory>
 #include "../basic/match.h"
 #include "../output/daa_file.h"
 #include "../output/daa_record.h"
@@ -28,6 +29,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../util/escape_sequences.h"
 #include "../basic/parameters.h"
 #include "../data/metadata.h"
+#include "../util/io/consumer.h"
 
 struct Output_format
 {
@@ -42,9 +44,9 @@ struct Output_format
 	{}
 	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out)
 	{}
-	virtual void print_header(OutputFile &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const
+	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const
 	{ }
-	virtual void print_footer(OutputFile &f) const
+	virtual void print_footer(Consumer &f) const
 	{ }
 	virtual Output_format* clone() const = 0;
 	virtual ~Output_format()
@@ -59,7 +61,7 @@ struct Output_format
 	enum { daa, blast_tab, blast_xml, sam, blast_pairwise, null, taxon, paf };
 };
 
-extern auto_ptr<Output_format> output_format;
+extern std::unique_ptr<Output_format> output_format;
 
 struct Null_format : public Output_format
 {
@@ -118,12 +120,12 @@ struct Sam_format : public Output_format
 	Sam_format():
 		Output_format(sam)
 	{ }
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out);
-	virtual void print_header(OutputFile &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const;
+	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
 	virtual ~Sam_format()
 	{ }
-	virtual Output_format* clone() const
+	virtual Output_format* clone() const override
 	{
 		return new Sam_format(*this);
 	}
@@ -136,14 +138,14 @@ struct XML_format : public Output_format
 	{
 		config.salltitles = true;
 	}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out);
-	virtual void print_header(OutputFile &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const;
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const;
-	virtual void print_footer(OutputFile &f) const;
+	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const override;
+	virtual void print_footer(Consumer &f) const override;
 	virtual ~XML_format()
 	{ }
-	virtual Output_format* clone() const
+	virtual Output_format* clone() const override
 	{
 		return new XML_format(*this);
 	}
@@ -156,14 +158,14 @@ struct Pairwise_format : public Output_format
 	{
 		config.salltitles = true;
 	}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out);
-	virtual void print_header(OutputFile &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const;
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const;
-	virtual void print_footer(OutputFile &f) const;
+	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const override;
+	virtual void print_footer(Consumer &f) const override;
 	virtual ~Pairwise_format()
 	{ }
-	virtual Output_format* clone() const
+	virtual Output_format* clone() const override
 	{
 		return new Pairwise_format(*this);
 	}
@@ -179,11 +181,11 @@ struct Taxon_format : public Output_format
 		needs_taxon_id_lists = true;
 		needs_taxon_nodes = true;
 	}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out);
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const;
+	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const override;
 	virtual ~Taxon_format()
 	{ }
-	virtual Output_format* clone() const
+	virtual Output_format* clone() const override
 	{
 		return new Taxon_format(*this);
 	}
