@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -21,6 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <iostream>
 #include <fstream>
+#include <limits.h>
 #include "Timer.h"
 #include "tinythread.h"
 
@@ -92,7 +93,7 @@ struct task_timer
 	}
 	void finish()
 	{
-		if (!msg_)
+		if (!msg_ || level_ == UINT_MAX)
 			return;
 		//if (print_ && !Cfg::debug_log)
 		get_stream() << " [" << timer_.getElapsedTimeInSec() << "s]" << endl;
@@ -123,7 +124,9 @@ private:
 	{
 		timer_.start();
 		//if (print_ && !Cfg::debug_log) {
-			get_stream() << msg << "... " << std::flush;
+		if (level_ == UINT_MAX)
+			return;
+		get_stream() << msg << "... " << std::flush;
 			//fflush(stdout);
 		/**}
 		else if (Cfg::debug_log)
