@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -192,13 +192,6 @@ private:
 	_t data_[d1][d2];
 };
 
-inline void auto_append_extension(string &str, const char *ext)
-{
-	size_t l = strlen(ext);
-	if(str.length() < l || (str.length() >= l && str.substr(str.length()-l, string::npos) != ext))
-		str += ext;
-}
-
 extern const char dir_separator;
 string extract_dir(const string &s);
 
@@ -378,13 +371,6 @@ bool equal(const _t *ptr, unsigned n)
 	return true;
 }
 
-inline string to_string(unsigned long long val)
-{
-	char buf[32];
-	sprintf(buf, "%llu", val);
-	return string(buf);
-}
-
 inline string print_char(char c)
 {
 	char buf[16];
@@ -540,4 +526,6 @@ inline set<unsigned> parse_csv(const string &s)
 	return r;
 }
 
+std::string join(char c, const std::vector<std::string> &v);
+
 #endif /* UTIL_H_ */
